@@ -30,6 +30,16 @@
 (defun machine-type nil)
 (defun machine-version nil)
 
+(defun short-site-name nil)
+(defun long-site-name nil)
+
+
+
+(defun describe (obj &optional str) (values))
+(defun describe-object (obj str))
+(defun inspect (obj))
+
+
 
 ;;; the following useless constants are just ridiculously high because lambda
 ;;; lists and multiple values use chained lists, so there's no hard limit
@@ -78,6 +88,9 @@
 (defun zerop (num)
   (= num 0))
 
+(defun signum (num)
+  (if (zerop num) num (/ num (abs num))))
+
 (defun mod (num div)
   (nth-value 1 (floor num div)))
 
@@ -96,6 +109,86 @@
 (defun conjugate (num)
   (complex (realpart num) (- (imagpart num))))
 
+(defun cis (rad)
+  (complex (cos rad) (sin rad)))
+
+
+(defun logand (&rest ints)
+  (lognot (apply 'logior (mapcar #'lognot ints))))
+
+(defun logandc1 (int1 int2)
+  (logand (lognot int1) int2))
+
+(defun logandc2 (int1 int2)
+  (logand int1 (lognot int2)))
+
+(defun logeqv (&rest ints)
+  (let ((l (length ints)))
+    (cond
+      ((= l 0) -1)
+      ((= l 1) (car ints))
+      ((= l 2) (lognot (logxor (car ints) (cadr ints))))
+      (t (logeqv (car ints) (apply 'logeqv (cdr ints)))))))
+
+(defun lognand (int1 int2)
+  (lognot (logand int1 int2)))
+
+(defun lognor (int1 int2)
+  (lognot (logior int1 int2)))
+
+(defun logorc1 (int1 int2)
+  (logior (lognot int1) int2))
+
+(defun logorc2 (int1 int2)
+  (logior int1 (lognot int2)))
+
+(defun logxor (&rest ints)
+  (let ((l (length ints)))
+    (cond
+      ((= l 0) 0)
+      ((= l 1) (car ints))
+      ((= l 2) (logand (logior (car ints) (cadr ints)) (lognot (logand (car ints) (cadr ints)))))
+      (t (logxor (car ints) (apply 'logxor (cdr ints)))))))
+
+
+(defconstant boole-1 'boole-1)
+(defconstant boole-2 'boole-2)
+(defconstant boole-andc1 'boole-andc1)
+(defconstant boole-andc2  'boole-andc2)
+(defconstant boole-and 'boole-and)
+(defconstant boole-c1 'boole-c1)
+(defconstant boole-c2 'boole-c2)
+(defconstant boole-clr 'boole-clr)
+(defconstant boole-eqv 'boole-eqv)
+(defconstant boole-ior 'boole-ior)
+(defconstant boole-nand 'boole-nand)
+(defconstant boole-nor 'boole-nor)
+(defconstant boole-orc1 'boole-orc1)
+(defconstant boole-orc2 'boole-orc2)
+(defconstant boole-set 'boole-set)
+(defconstant boole-xor 'boole-xor)
+
+(defun boole (op int1 int2)
+  (cond
+    ((eq op 'boole-1) int1)
+    ((eq op 'boole-2) int2)
+    ((eq op 'boole-andc1) (logandc1 int1 int2))
+    ((eq op 'boole-andc2) (logandc2 int1 int2))
+    ((eq op 'boole-and) (logand int1 int2))
+    ((eq op 'boole-c1) (lognot int1))
+    ((eq op 'boole-c2) (lognot int2))
+    ((eq op 'boole-clr) 0)
+    ((eq op 'boole-eqv) (logeqv int1 int2))
+    ((eq op 'boole-ior) (logior int1 int2))
+    ((eq op 'boole-nand) (lognand int1 int2))
+    ((eq op 'boole-nor) (lognor int1 int2))
+    ((eq op 'boole-orc1) (logorc1 int1 int2))
+    ((eq op 'boole-orc2) (logorc2 int1 int2))
+    ((eq op 'boole-set) -1)
+    ((eq op 'boole-xor) (logxor int1 int2))
+    (t (+ ""))))
+
+
 
 (defparameter *gensym-counter* 1)
 
@@ -113,6 +206,20 @@
     (make-symbol (get-output-stream-string str))))
 
 
+(defparameter gentemp-counter 1)
+
+(defun gentemp (&optional (pr "T") (pack *package*))
+  (let ((str (make-string-output-stream)))
+    (loop
+     (write-string pr str)
+     (write gentemp-counter :stream str)
+     (incf gentemp-counter)
+     (let ((s (get-output-stream-string str)))
+       (if (not (find-symbol s pack))
+	   (return-from gentemp (values (intern s pack))))))))
+
+
+
 (defun first (list) (nth 0 list))
 (defun second (list) (nth 1 list))
 (defun third (list) (nth 2 list))
@@ -158,9 +265,41 @@
 (defun cddddr (list) (nthcdr 4 list))
 
 
+(defun make-list (len &key initial-element)
+  (let (out)
+    (dotimes (i len)
+      (setq out (cons initial-element out)))
+    out))
+
+
 (defun endp (l) (null l))
 
 
+(defun butlast (ls &optional (n 1))
+  (let* ((l (length ls))
+	 (last (max 0 (- l n)))
+	 out)
+    (dotimes (i last)
+      (setq out (cons (car ls) out))
+      (setq ls (cdr ls)))
+    (reverse out)))
+
+
+(defun nbutlast (ls &optional (n 1))
+  (butlast ls n))
+
+
+(defun acons (key dat list)
+  (cons (cons key dat) list))
+
+
+(defun pairlis (keys data &optional alist)
+  (dolist (k keys)
+    (setq alist (cons (cons k (car data)) alist))
+    (setq data (cdr data)))
+  alist)
+
+
 
 (define-setf-expander symbol-value (sym)
   (let ((var (gensym))
@@ -416,6 +555,27 @@
 	  (array-dimensions array))))
 
 
+(defun get (sym ind &optional (def nil))
+  (let* ((pl (symbol-plist sym))
+	 (l (length pl)))
+    (do ((i 0 (+ i 2)))
+	((>= i l))
+      (if (eq (car pl) ind)
+	  (return-from get (cadr pl)))
+      (setq pl (cddr pl))))
+  def)
+
+
+(defun get-properties (pl il)
+  (let ((l (length pl)))
+    (do ((i 0 (+ i 2)))
+	((>= i l))
+      (if (position-if (lambda (e) (eq e (car pl))) il)
+	  (return-from get-properties (values (car pl) (cadr pl) pl)))
+      (setq pl (cddr pl))))
+  (values nil nil nil))
+
+
 (defun char (str ind)
   (aref str ind))
 
@@ -577,6 +737,9 @@
 (defun complexp (obj)
   (typep obj 'complex))
 
+(defun random-state-p (obj)
+  (typep obj 'random-state))
+
 (defun characterp (obj)
   (typep obj 'character))
 
@@ -722,87 +885,85 @@
 	finally-forms
 	do-forms
 	vars var)
-    (do ((forms forms (cdr forms)))
+    (do ()
 	((not forms))
       (let ((form (car forms)))
 	(if (atom form)
 	    (let ((sym (string form)))
 	      (cond
-                 ((string= sym "INITIALLY")
-		 (do ((f (cdr forms)))
-		     ((atom (car f)))
-		   (setq initially-forms (append initially-forms `(,(car f))))
-		   (setq f (cdr f))))
+                ((string= sym "INITIALLY")
+		 (do ((f (cdr forms) (cdr f)))
+		     ((atom (car f)) (setq forms f))
+		   (setq initially-forms (append initially-forms `(,(car f))))))
 		((string= sym "FINALLY")
-		 (do ((f (cdr forms)))
-		     ((atom (car f)))
-		   (setq finally-forms (append finally-forms `(,(car f))))
-		   (setq f (cdr f))))
+		 (do ((f (cdr forms) (cdr f)))
+		     ((atom (car f)) (setq forms f))
+		   (setq finally-forms (append finally-forms `(,(car f))))))
 		((or (string= sym "DO")
 		     (string= sym "DOING"))
-		 (do ((f (cdr forms)))
-		     ((atom (car f)))
-		   (setq do-forms (append do-forms `(,(car f))))
-		   (setq f (cdr f))))
+		 (do ((f (cdr forms) (cdr f)))
+		     ((atom (car f)) (setq forms f))
+		   (setq do-forms (append do-forms `(,(car f))))))
 		((string= sym "NAMED")
 		 (setq block-name (cadr forms))
-		 (setq forms (cdr forms)))
+		 (setq forms (cddr forms)))
 		((or (string= sym "FOR")
 		     (string= sym "AS"))
-		 (setq forms (cdr forms))
-		 (setq var `(,(car forms) 0 nil 1 1))
-		 (setq vars (append vars `(,var))))
+		 (setq var `(,(cadr forms) 0 nil 1 1))
+		 (setq vars (append vars `(,var)))
+		 (setq forms (cddr forms)))
 		((string= sym "FROM")
-		 (setq forms (cdr forms))
-		 (setf (elt var 1) (car forms)))
+		 (setf (elt var 1) (cadr forms))
+		 (setq forms (cddr forms)))
 		((or (string= sym "DOWNFROM"))
-		 (setq forms (cdr forms))
-		 (setf (elt var 1) (car forms))
-		 (setf (elt var 3) -1))
+		 (setf (elt var 1) (cadr forms))
+		 (setf (elt var 3) -1)
+		 (setq forms (cddr forms)))
 		((or (string= sym "UPFROM"))
-		 (setq forms (cdr forms))
-		 (setf (elt var 1) (car forms))
-		 (setf (elt var 3) 1))
+		 (setf (elt var 1) (cadr forms))
+		 (setf (elt var 3) 1)
+		 (setq forms (cddr forms)))
 		((or (string= sym "TO"))
-		 (setq forms (cdr forms))
-		 (setf (elt var 2) (car forms)))
+		 (setf (elt var 2) (cadr forms))
+		 (setq forms (cddr forms)))
 		((or (string= sym "DOWNTO"))
-		 (setq forms (cdr forms))
-		 (setf (elt var 2) (car forms))
-		 (setf (elt var 3) -1))
+		 (setf (elt var 2) (cadr forms))
+		 (setf (elt var 3) -1)
+		 (setq forms (cddr forms)))
 		((or (string= sym "UPTO"))
-		 (setq forms (cdr forms))
-		 (setf (elt var 2) (car forms))
-		 (setf (elt var 3) 1))
+		 (setf (elt var 2) (cadr forms))
+		 (setf (elt var 3) 1)
+		 (setq forms (cddr forms)))
 		((or (string= sym "BELOW"))
-		 (setq forms (cdr forms))
-		 (setf (elt var 2) (1- (car forms))))
+		 (setf (elt var 2) (1- (cadr forms)))
+		 (setq forms (cddr forms)))
 		((or (string= sym "ABOVE"))
-		 (setq forms (cdr forms))
-		 (setf (elt var 2) (1+ (car forms))))
+		 (setf (elt var 2) (1+ (cadr forms)))
+		 (setq forms (cddr forms)))
 		((or (string= sym "BY"))
-		 (setq forms (cdr forms))
-		 (setf (elt var 4) (car forms)))))
-	    (append do-forms `(,form)))))
+		 (setf (elt var 4) (cadr forms))
+		 (setq forms (cddr forms)))))
+	    (progn
+	      (setq do-forms (append do-forms `(,form)))
+	      (setq forms (cdr forms))))))
     `(block ,block-name
-       (tagbody
-	  (let ,(mapcar (lambda (x) `(,(car x) ,(cadr x))) vars)
-	    ,@initially-forms
-	    (block ,inner-block-name
-	      (tagbody
-		 ,tagname
-		 (if (or ,@(mapcar (lambda (x) (if (elt x 2)
-						      (if (= (elt x 3) 1)
-							  `(> ,(elt x 0) ,(elt x 2))
-							  `(< ,(elt x 0) ,(elt x 2)))
-						      nil)) vars))
-		     (return-from ,inner-block-name nil))
-		 ,@do-forms
-		 (progn
-		   ,@(mapcar (lambda (x) `(setq ,(elt x 0) (+ ,(elt x 0) ,(* (elt x 3) (elt x 4))))) vars))
-		 (go ,tagname)))
-	    ,@finally-forms
-	    nil)))))
+       (let ,(mapcar (lambda (x) `(,(car x) ,(cadr x))) vars)
+	 ,@initially-forms
+	 (block ,inner-block-name
+	   (tagbody
+	      ,tagname
+	      (if (or ,@(mapcar (lambda (x) (if (elt x 2)
+						(if (= (elt x 3) 1)
+						    `(> ,(elt x 0) ,(elt x 2))
+						    `(< ,(elt x 0) ,(elt x 2)))
+						nil)) vars))
+		  (return-from ,inner-block-name nil))
+	      ,@do-forms
+	      (progn
+		,@(mapcar (lambda (x) `(setq ,(elt x 0) (+ ,(elt x 0) ,(* (elt x 3) (elt x 4))))) vars))
+	      (go ,tagname)))
+	 ,@finally-forms
+	 nil))))
 
 
 (defun format (out fstr &rest args)
@@ -825,30 +986,49 @@
 
 
 
+(defun encode-universal-time (sec min hour d m y &optional tz)
+  (+ (* (- y 1900) 31536000)
+     (* 86400 (+ (1+ (floor (- y 1900 1) 4))
+		 (- (1+ (floor (- y 1901) 100)))
+		 (floor (- y 1601))))
+     (* 86400 (nth m '(0 0 31 59 90 120 151 181 212 243 273 304 334)))
+     (* (1- d) 86400)
+     (* hour 3600)
+     (* min 60)
+     sec))
+
+
+
 
 (export '(*features* machine-instance machine-type machine-version
+	  short-site-name long-site-name describe describe-object inspect
 	  lambda-parameters-limit call-arguments-limit multiple-values-limit
 	  array-rank-limit array-dimension-limit array-total-size-limit
 	  char-code-limit lambda-list-keywords identity pi 1+ 1- minusp plusp
-	  abs zerop mod rem evenp oddp isqrt conjugate *gensym-counter* gensym
-	  first second third fourth fifth sixth seventh eighth ninth tenth rest
-	  caar cadr cdar cddr caaar caadr cadar caddr cdaar cdadr cddar cdddr
-	  caaaar caaadr caadar caaddr cadaar cadadr caddar cadddr cdaaar cdaadr
-	  cdadar cdaddr cddaar cddadr cdddar cddddr endp when unless incf decf
-	  and or cond otherwise case return multiple-value-bind every some
-	  notany notevery member member-if member-if-not find find-if
-	  find-if-not assoc assoc-if assoc-if-not position position-if
-	  position-if-not count count-if count-if-not remove remove-if-not
-	  delete delete-if delete-if-not nreverse adjoin fill push pop
-	  array-rank array-dimension array-total-size array-in-bounds-p char
-	  string/= char-equal digit-char digit-char-p char-int string-upcase
-	  string-downcase string-capitalize nstring-upcase nstring-downcase
-	  nstring-capitalize string-left-trim string-right-trim string-trim
-	  defpackage consp listp symbolp keywordp functionp packagep integerp
-	  rationalp floatp complexp characterp vectorp arrayp sequencep stringp
+	  abs zerop signum mod rem evenp oddp isqrt conjugate cis logand
+	  logandc1 logandc2 logeqv lognand lognor logorc1 logorc2 logxor boole-1
+	  boole-2 boole-andc1 boole-andc2 boole-and boole-c1 boole-c2 boole-clr
+	  boole-eqv boole-ior boole-nand boole-nor boole-orc1 boole-orc2
+	  boole-set boole-xor boole *gensym-counter* gensym gentemp first second
+	  third fourth fifth sixth seventh eighth ninth tenth rest caar cadr
+	  cdar cddr caaar caadr cadar caddr cdaar cdadr cddar cdddr caaaar
+	  caaadr caadar caaddr cadaar cadadr caddar cadddr cdaaar cdaadr cdadar
+	  cdaddr cddaar cddadr cdddar cddddr make-list endp butlast nbutlast
+	  acons pairlis when unless incf decf and or cond otherwise case return
+	  multiple-value-bind every some notany notevery member member-if
+	  member-if-not find find-if find-if-not assoc assoc-if assoc-if-not
+	  position position-if position-if-not count count-if count-if-not
+	  remove remove-if-not delete delete-if delete-if-not nreverse adjoin
+	  fill push pop array-rank array-dimension array-total-size
+	  array-in-bounds-p get get-properties char string/= char-equal
+	  digit-char digit-char-p char-int string-upcase string-downcase
+	  string-capitalize nstring-upcase nstring-downcase nstring-capitalize
+	  string-left-trim string-right-trim string-trim defpackage consp listp
+	  symbolp keywordp functionp packagep integerp rationalp floatp complexp
+	  random-state-p characterp vectorp arrayp sequencep stringp
 	  hash-table-p pathnamep streamp realp numberp macroexpand equal equalp
 	  fdefinition complement mapc terpri write-line write-sequence prin1
-	  princ print loop format))
+	  princ print loop format encode-universal-time))
 
 
 
