@@ -348,6 +348,8 @@ make_test ("(f :a 0)", "NIL");
 make_test ("(defun f (&key x &allow-other-keys) x)", "F");
 make_test ("(f :x 0 :y 1)", "0");
 make_test ("(f)", "NIL");
+make_test ("(defun f (x y &aux (z (+ x y)) w) (list z w))", "F");
+make_test ("(f 1 2)", "(3 NIL)");
 make_test ("(let ((x 0)) (defun inc () (setf x (+ x 1))))", "INC");
 make_test ("(inc)", "1");
 make_test ("(inc)", "2");
@@ -402,6 +404,7 @@ make_test ("(typep 0.1 'short-float)", "T");
 make_test ("(typep 0.1 'long-float)", "T");
 make_test ("(typep 1.0 'complex)", "NIL");
 make_test ("(typep (complex 1 1) 'complex)", "T");
+make_test ("(typep (make-random-state) 'random-state)", "T");
 make_test ("(typep \"abc\" 'string)", "T");
 make_test ("(typep \"abc\" '(string))", "T");
 make_test ("(typep \"abc\" 'array)", "T");
@@ -438,11 +441,25 @@ make_test ("(type-of #(0 1 2))", "ARRAY");
 make_test ("(type-of (make-hash-table))", "HASH-TABLE");
 make_test ("(type-of #p\"aaa\")", "PATHNAME");
 make_test ("(type-of *standard-output*)", "STREAM");
+make_test ("(subtypep 'integer nil)", "NIL\nT");
+make_test ("(subtypep 'integer t)", "T\nT");
+make_test ("(subtypep nil 'integer)", "T\nT");
+make_test ("(subtypep t 'integer)", "NIL\nT");
+make_test ("(subtypep 'integer 'number)", "T\nT");
+make_test ("(subtypep 'integer 'cons)", "NIL\nT");
+make_test ("(subtypep 'short-float 'long-float)", "T\nT");
+make_test ("(subtypep 'long-float 'short-float)", "T\nT");
+make_test ("(subtypep 'long-float 'single-float)", "T\nT");
 make_test ("(deftype not-integer () `(not integer))", "NOT-INTEGER");
 make_test ("(typep 0 'not-integer)", "NIL");
 make_test ("(typep \"\" 'not-integer)", "T");
 make_test ("(define-setf-expander foo ())", "FOO");
 make_test ("(get-setf-expansion '(foo))", "NIL");
+make_test ("(defun (setf foo2) (a1 a2) (list a1 a2))", "(SETF FOO2)");
+make_test ("#'(setf foo2)", "#<FUNCTION (SETF FOO2)>");
+make_test ("(fboundp '(setf foo2))", "T");
+make_test ("(fboundp '(setf foo3))", "NIL");
+make_test ("(let ((x 0) (y 1)) (setf (foo2 x) y))", "(1 0)");
 
 make_test ("(make-string 3)", "\"\0\0\0\"");
 make_test ("(intern \"hi\")", "|hi|\nNIL");
@@ -483,6 +500,13 @@ make_test ("(symbol-package 'ggg)", "#<PACKAGE \"COMMON-LISP-USER\">");
 make_test ("(symbol-package :eee)", "#<PACKAGE \"KEYWORD\">");
 make_test ("(symbol-package '#:aaa)", "NIL");
 make_test ("(symbol-package (make-symbol \"aaa\"))", "NIL");
+make_test ("(symbol-plist 's)", "NIL");
+make_test ("(setf (symbol-plist 's) (list :a 0 :b 1))", "(:A 0 :B 1)");
+make_test ("(symbol-plist 's)", "(:A 0 :B 1)");
+make_test ("(get 's :b)", "1");
+make_test ("(get-properties (symbol-plist 's) '(:b :a))", ":A\n0\n(:A 0 :B 1)");
+make_test ("(get-properties (symbol-plist 's) '(:b))", ":B\n1\n(:B 1)");
+make_test ("(get-properties (symbol-plist 's) '(:c))", "NIL\nNIL\nNIL");
 make_test ("(special-operator-p 'if)", "T");
 make_test ("(special-operator-p 'car)", "NIL");
 make_test ("(special-operator-p 'aaa)", "NIL");
@@ -690,6 +714,7 @@ make_test ("(princ '|Aa,a|)", "Aa,a\n|Aa,a|");
 make_test ("(princ #\\a)", "a\n#\\a");
 make_test ("(princ \"aaa\" *standard-output*)", "aaa\n\"aaa\"");
 make_test ("(print \"aaa\" *standard-output*)", "\n\"aaa\" \n\"aaa\"");
+make_test ("(let ((c 0)) (loop (if (= c 2) (return 0)) (incf c)))", "0");
 make_test ("(loop initially (write 'hello) for i from 1 to 5 finally (write 'world) do (write i))", "HELLO12345WORLD\nNIL");
 make_test ("(loop for i from 1 to 5 do (write i))", "12345\nNIL");
 make_test ("(loop for i from 1 below 5 do (write i))", "1234\nNIL");
@@ -725,15 +750,29 @@ make_test ("(write 100 :stream str)", "100");
 make_test ("(write 1.2 :stream str)", "1.2");
 make_test ("(get-output-stream-string str)", "\"AAA\\\"bbb\\\"CCC#\\\\c1001.2\"");
 make_test ("(get-output-stream-string str)", "\"\"");
-make_test ("(gensym)", "#:G19");
-make_test ("(gensym)", "#:G20");
-make_test ("(gensym 5)", "#:G5");
-make_test ("(gensym \"A\")", "#:A21");
+make_test ("(gensym)", "#:G21");
 make_test ("(gensym)", "#:G22");
+make_test ("(gensym 5)", "#:G5");
+make_test ("(gensym \"A\")", "#:A23");
+make_test ("(gensym)", "#:G24");
+make_test ("(gentemp)", "T1");
+make_test ("(gentemp \"S\")", "S2");
+make_test ("'s3", "S3");
+make_test ("(gentemp \"S\")", "S4");
 make_test ("(second '(0))", "NIL");
 make_test ("(fifth '(0 1 2 3 4))", "4");
+make_test ("(make-list 3)", "(NIL NIL NIL)");
+make_test ("(make-list 4 :initial-element 0)", "(0 0 0 0)");
 make_test ("(endp '(1 . 2))", "NIL");
 make_test ("(endp ())", "T");
+make_test ("(butlast '(0 1 2))", "(0 1)");
+make_test ("(butlast '(0 1 2) 3)", "NIL");
+make_test ("(butlast '(0 1 2 . 3) 2)", "(0)");
+make_test ("(nbutlast '(0 1 2))", "(0 1)");
+make_test ("(nbutlast '(0 1 2) 3)", "NIL");
+make_test ("(nbutlast '(0 1 2 . 3) 2)", "(0)");
+make_test ("(acons 1 \"\" '((2 . \"\")))", "((1 . \"\") (2 . \"\"))");
+make_test ("(pairlis '(1 2 3) '(\"a\" \"b\" \"c\") '((4 . \"d\")))", "((3 . \"c\") (2 . \"b\") (1 . \"a\") (4 . \"d\"))");
 make_test ("(when t \"\" \"\" \"\")", "\"\"");
 make_test ("(when nil \"\" \"\" \"\")", "NIL");
 make_test ("(unless nil \"\" \"\" \"\")", "\"\"");
@@ -749,12 +788,30 @@ make_test ("#'make-ship", "#<FUNCTION MAKE-SHIP>");
 make_test ("#'ship-x", "#<FUNCTION SHIP-X>");
 make_test ("(defparameter s1 (make-ship))", "S1");
 make_test ("s1", "#<STRUCTURE OF CLASS SHIP>");
+make_test ("(typep s1 'ship)", "T");
+make_test ("(typep 0 'ship)", "NIL");
 make_test ("(ship-x s1)", "NIL");
 make_test ("(ship-y s1)", "NIL");
 make_test ("(setf (ship-x s1) 0)", "0");
 make_test ("(setf (ship-y s1) 1)", "1");
 make_test ("(ship-x s1)", "0");
 make_test ("(ship-y s1)", "1");
+make_test ("(defclass starship () (x y speed))", "#<STANDARD-CLASS STARSHIP>");
+make_test ("(find-class 'starship)", "#<STANDARD-CLASS STARSHIP>");
+make_test ("(defparameter inst (make-instance 'starship))", "INST");
+make_test ("inst", "#<STARSHIP OBJECT ...>");
+make_test ("(class-of inst)", "#<STANDARD-CLASS STARSHIP>");
+make_test ("(setf (slot-value inst 'x) 0)", "0");
+make_test ("(setf (slot-value inst 'y) 2.5)", "2.5");
+make_test ("(slot-value inst 'x)", "0");
+make_test ("(slot-value inst 'y)", "2.5");
+make_test ("(defgeneric genfun (x y z))", "#<STANDARD-GENERIC-FUNCTION GENFUN>");
+make_test ("(defmethod genfun ((x string) y z) 10)", "#<STANDARD-METHOD GENFUN>");
+make_test ("(defmethod genfun ((x integer) y z) 11)", "#<STANDARD-METHOD GENFUN>");
+make_test ("(genfun \"\" nil nil)", "10");
+make_test ("(genfun 0 nil nil)", "11");
+make_test ("(define-condition new-math-error (arithmetic-error) (a b))", "NEW-MATH-ERROR");
+make_test ("(make-condition 'new-math-error)", "#<CONDITION OF CLASS NEW-MATH-ERROR>");
 make_test ("(tagbody\n" .
 	   "  (write \"1\")\n" .
 	   "  (go jmp)\n" .
@@ -768,6 +825,7 @@ make_test ("(block test (write 1) (write 2) (return-from test 10) (write 3))", "
 make_test ("(block test (write 1) (block test2 (write 2) (return-from test 10) (write 3)))", "12\n10");
 make_test ("(block test (write 1) (block test2 (write 2) (return-from test (values 10 11)) (write 3)))", "12\n10\n11");
 make_test ("(block nil (return (values 11 12)))", "11\n12");
+make_test ("(block b (do nil (nil) (return-from b (values 0 1 2))))", "0\n1\n2");
 make_test ("(defun test () (write 1) (write 2) (return-from test 10) (write 3))", "TEST");
 make_test ("(test)", "12\n10");
 make_test ("(defun f () (throw 'label (values 0 1 2)))", "F");
@@ -775,6 +833,8 @@ make_test ("(catch 'label (f))", "0\n1\n2");
 make_test ("(block nil (handler-bind ((file-error (lambda (e) (write 0))) (division-by-zero (lambda (e) (return 1)))) (/ 1 0)))", "1");
 make_test ("(block nil (handler-bind ((division-by-zero (lambda (e) (write 0))) (arithmetic-error (lambda (e) (write 1))) (error (lambda (e) (return 2)))) (/ 1 0)))", "01\n2");
 make_test ("(block nil (handler-bind ((division-by-zero (lambda (e) (return 0)))) (handler-bind ((arithmetic-error (lambda (e) (write 1)))) (/ 1 0))))", "1\n0");
+make_test ("(block nil (handler-bind ((type-error (lambda (e) (write 0) (return 1)))) (+ \"\")))", "0\n1");
+make_test ("(restart-bind ((rest (lambda () (write 10)))) (invoke-restart 'rest) (write 11))", "1011\n11");
 make_test ("(block test (unwind-protect 0 (write 'whatever)))", "WHATEVER\n0");
 make_test ("(block test (unwind-protect (return-from test 0) (write 'whatever)))", "WHATEVER\n0");
 make_test ("(block test (unwind-protect (return-from test (values 0 1)) (write 'whatever)))", "WHATEVER\n0\n1");
@@ -785,6 +845,7 @@ make_test ("(caar '((4) 1 2 3))", "4");
 make_test ("(incf c)", "6");
 make_test ("(incf c 3)", "9");
 make_test ("(let ((i 0)) (incf i))", "1");
+make_test ("(let ((arr (make-array 2)) (x 0)) (setf (aref arr (incf x)) (incf x)) arr)", "#(NIL 2)");
 make_test ("(decf c)", "8");
 make_test ("(decf c c)", "0");
 make_test ("(and)", "T");
@@ -882,9 +943,12 @@ make_test ("(count #\\a \"abcabc\")", "2");
 make_test ("(count 2 '(1 2 3))", "1");
 make_test ("(count-if #'upper-case-p \"aAbBcCdD\")", "4");
 make_test ("(count-if-not #'evenp #(0 1 2))", "1");
+make_test ("(>= (get-internal-run-time) 0)", "T");
 make_test ("(machine-type)", "NIL");
 make_test ("(machine-instance)", "NIL");
 make_test ("(machine-version)", "NIL");
+make_test ("(short-site-name)", "NIL");
+make_test ("(long-site-name)", "NIL");
 
 make_test ("#+common-lisp \"\" \"\"", "\"\"\n\"\"");
 make_test ("#-common-lisp \"\" \"\"", "\"\"");
@@ -1027,6 +1091,7 @@ make_test ("(imagpart 1)", "0");
 make_test ("(imagpart (complex 1 1/2))", "1/2");
 make_test ("(conjugate 1)", "1");
 make_test ("(conjugate (complex 1 1))", "#C(1 -1)");
+make_test ("(cis 1.5)", "#C(0.0707372016677029064047 0.9974949866040544455)");
 make_test ("(= 1)", "T");
 make_test ("(= 1 1)", "T");
 make_test ("(= 1 2)", "NIL");
@@ -1049,6 +1114,56 @@ make_test ("(min 0 1/2 -0.1)", "-0.1");
 make_test ("(max 0)", "0");
 make_test ("(max 0 1.0 2)", "2");
 make_test ("(max 0 1/2 -0.1)", "1/2");
+make_test ("(sin 0)", "0.0");
+make_test ("(sin 0.5)", "0.479425538604203005377");
+make_test ("(cos 1/2)", "0.877582561890372758739");
+make_test ("(tan 2.5)", "-0.747022297238660315521");
+make_test ("(sinh 10)", "11013.2328747033934633");
+make_test ("(cosh 1/2)", "1.12762596520638069819");
+make_test ("(tanh 3.5)", "0.998177897611198683414");
+make_test ("(exp 0)", "1.0");
+make_test ("(exp 2.5)", "12.1824939607034732347");
+make_test ("(expt 2 3)", "8");
+make_test ("(expt 2 0)", "1");
+make_test ("(expt 1/2 -2)", "4");
+make_test ("(expt 2 1/2)", "1.41421356237309514547");
+make_test ("(expt 1.2 -2.5)", "0.633938145260608987286");
+make_test ("(log 1)", "0.0");
+make_test ("(log 2.5)", "0.916290731874155106595");
+make_test ("(log 1 0)", "0.0");
+make_test ("(log 2.5 1.2)", "5.02568510266547630039");
+make_test ("(lognot 1024)", "-1025");
+make_test ("(lognot -1025)", "1024");
+make_test ("(logior)", "0");
+make_test ("(logior -1)", "-1");
+make_test ("(logior 0 1 -4 27 259)", "-1");
+make_test ("(logand)", "-1");
+make_test ("(logand 15 7)", "7");
+make_test ("(logand 1 2 4 129 -367)", "0");
+make_test ("(logandc1 27 -49)", "-60");
+make_test ("(logandc2 27 -49)", "16");
+make_test ("(logeqv 1 2 3 4 13)", "9");
+make_test ("(lognand 27 -49)", "-12");
+make_test ("(lognor 27 -49)", "32");
+make_test ("(logorc1 27 -49)", "-17");
+make_test ("(logorc2 27 -49)", "59");
+make_test ("(logxor 1 2 3 4 13)", "9");
+make_test ("(boole boole-1 3 -7)", "3");
+make_test ("(boole boole-2 3 -7)", "-7");
+make_test ("(boole boole-andc1 3 -7)", "-8");
+make_test ("(boole boole-andc2 3 -7)", "2");
+make_test ("(boole boole-and 3 -7)", "1");
+make_test ("(boole boole-c1 3 -7)", "-4");
+make_test ("(boole boole-c2 3 -7)", "6");
+make_test ("(boole boole-clr 3 -7)", "0");
+make_test ("(boole boole-eqv 3 -7)", "5");
+make_test ("(boole boole-ior 3 -7)", "-5");
+make_test ("(boole boole-nand 3 -7)", "-2");
+make_test ("(boole boole-nor 3 -7)", "4");
+make_test ("(boole boole-orc1 3 -7)", "-3");
+make_test ("(boole boole-orc2 3 -7)", "7");
+make_test ("(boole boole-set 3 -7)", "-1");
+make_test ("(boole boole-xor 3 -7)", "-6");
 make_test ("(minusp 0)", "NIL");
 make_test ("(minusp -0.5)", "T");
 make_test ("(plusp 1)", "T");
@@ -1058,6 +1173,9 @@ make_test ("(abs -0.5)", "0.5");
 make_test ("(abs 1/2)", "1/2");
 make_test ("(zerop -0)", "T");
 make_test ("(zerop 1/2)", "NIL");
+make_test ("(signum 0)", "0");
+make_test ("(signum 3.5)", "1.0");
+make_test ("(signum -1/2)", "-1");
 make_test ("(mod 2.5 1)", "0.5");
 make_test ("(mod -2.5 1)", "0.5");
 make_test ("(rem 2.5 1)", "0.5");
@@ -1066,6 +1184,14 @@ make_test ("(evenp 0)", "T");
 make_test ("(evenp 1)", "NIL");
 make_test ("(oddp 0)", "NIL");
 make_test ("(oddp -1)", "T");
+make_test ("*random-state*", "#<RANDOM-STATE ?>");
+make_test ("(make-random-state)", "#<RANDOM-STATE ?>");
+make_test ("(make-random-state t)", "#<RANDOM-STATE ?>");
+make_test ("(make-random-state nil)", "#<RANDOM-STATE ?>");
+make_test ("(make-random-state *random-state*)", "#<RANDOM-STATE ?>");
+make_test ("(< (random 10) 10)", "T");
+make_test ("(< (random 4.5) 4.5)", "T");
+make_test ("(< (random 10.5 (make-random-state)) 10.5)", "T");
 
 
 
